/*
 * utilities.c
 *
 * Created on: Jan 6, 2010
 * Author: fwuerri
 */

//-----------------------------------------------------------------------------
// includes

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "pm_at32ap7000.h"
#include "board.h"
#include "com.h"
#include "global.h"
#include "error.h"

//-----------------------------------------------------------------------------
// definitions

#define kYokto	'y'
#define kZepto	'z'
#define kAtto	'a'
#define kFemto	'f'
#define kPico	'p'
#define kNano	'n'
#define kMicro	'u'
#define kMilli	'm'
#define kCenti	'c'
#define kDeci	'd'
#define kDeca	'da'
#define kHekto	'h'
#define kKilo	'k'
#define kMega	'M'
#define kGiga 	'G'
#define kTera 	'T'
#define kPeta 	'P'
#define kExa 	'E'
#define kZetta 	'Z'
#define kYotta 	'Y'

extern long	gDate;



void RemoveTrailingZeros( char *string )
{
	char	*p, *p1;
	int		len;


	// remove trailing zeros

	len = strlen( string );
	p = string + len;

	//	sprintf( message, "%s; len = %d; string = %p; p = %p\r", string, len, string, p );
	//	PrintSCI0( message );

	// find the end of the number in string
	while(!isdigit( *p ) && p >= string) p--;
	if (p < string) return;							// no digit found

	// remove zero digits
	while (*p == '0' && p >= string)
	{
		p1 = p;
		while (*p1 != '\0')
		{
			*p1 = *(p1 + 1);
			p1++;
		}
		p--;
	}

	// remove a dot, if last char
	if (*p == '.')
	{
		p1 = p;
		while (*p1 != '\0')
		{
			*p1 = *(p1 + 1);
			p1++;
		}
	}

	// remove leading spaces
	p = string;
	p1 = string;
	while (*p && (*p == ' ' || *p == '\t')) p++;
	if (p != p1)
	{
		while (*p) *p1++ = *p++;
		*p1 = '\0';
	}

	// remove spaces between minus and number
	p = string;
	while (*p && *p != '-') p++;
	p1 = ++p;
	while (*p && (*p == ' ' || *p == '\t')) p++;
	if (p != p1)
	{
		while (*p) *p1++ = *p++;
		*p1 = '\0';
	}
}

//-----------------------------------------------------------------------------
void DivString( int shift, int trailing, char *string )
{
	int				len, cnt = 0, diff;
	char			*p, *p1, *pd;


	if (shift == 0) return;

	len = strlen( string );
	p = string + len;

//	Printf( "p = %p; string = %p; len = %d\n", &p, &string, len );
	// find the end of the number in string
	while(!isdigit( *p ) && p >= string) p--;
	if (p < string) return;							// no digit found

	// count the number of digits
	while(p >= string && isdigit( *p-- )) cnt++;

	p = string + len;

	diff = shift - cnt;
	if (diff < 0) diff = 0;
	diff++;

	pd = p + diff;
//	Printf( "p = %p; pd = %p; diff = %d\n", &p, &pd, diff );
	// make place for the dot and zeros if needed
	while (p >= string && !isdigit( *p ))
	{
		*pd-- = *p--;
	}

	// now shift the digits
	while (shift && p >= string && isdigit( *p ))
	{
		*pd-- = *p--;
		shift--;
	}

	p += diff;

	// do we need to fill zeros?
	if (shift > 0)
	{
		while (p > string && shift--)
		{
			*p-- = '0';
		}
	}
	*p = '.';

	// if the dot is the beginning, fill in a zero
	if (p == string || !isdigit( *(p - 1)))
	{
		len = strlen( string );
		p1 = string + len + 1;
		do
		{
			p1--;
			*(p1 + 1) = *p1;
		} while (p1 >= p);
		*p = '0';
	}

	RemoveTrailingZeros( string );

	// remove unused digits

	len = strlen( string );
	p = string + len;

	// find the end of the number in string
	while(!isdigit( *p ) && p >= string) p--;
	if (p <= string) return;						// no digit found

	cnt = 0;
	pd = p;
	// find the dot
	while(*pd != '.' && pd >= string)
	{
		pd--;
		cnt++;
	}
	if (trailing == 0 || cnt <= trailing) return;	// nothing to do
	pd = pd + trailing + 1;
	p++;

	while (*p != '\0') *pd++ = *p++;
	*pd = *p;
}

//-----------------------------------------------------------------------------
// time in 20us units

int MakeTimeString( char *buf, unsigned long time )
{
	unsigned long	hours, minutes, seconds, t;
	short			len;
	char			*p, *p1;
	char			string[20], noDot = 0;

	if (buf == NULL) return -1;
	*buf = '\0';

	hours = time / 180000000UL;
	if (hours) time -= hours * 180000000UL;
	minutes = time / 3000000UL;
	if (minutes) time -= minutes * 3000000UL;
	seconds = time / 50000UL;
	if (seconds) time -= seconds * 50000UL;

	// now make microseconds from the rest
	time *= 20ULL;

	if (hours)
	{
		sprintf( string, "%lu:", hours );
		strcat( buf, string );
	}

	if (minutes)
	{
		sprintf( string, "%02lu:", minutes );
		strcat( buf, string );
	}
	else
	{
		if (hours)
		{
			strcat( buf, "00:" );
		}
	}

	if (seconds)
	{
		if (hours || minutes)
		{
			sprintf( string, "%02lu", seconds );
		}
		else
		{
			sprintf( string, "%lu", seconds );
		}
		strcat( buf, string );
	}
	else
	{
		if (hours || minutes)
		{
			strcat( buf, "00" );
		}
	}

	if (time)
	{
		if (hours || minutes)
		{
			sprintf( string, ".%06lu", time );
			strcat( buf, string );
		}
		else if (seconds)
		{
			sprintf( string, ".%06lu s", time );
			strcat( buf, string );
		}
		else
		{
			noDot = 1;
			if (time >= 1000L)
			{
				t = time / 1000;
				if ((t * 1000) == time)
				{
					sprintf( string, "%lu ms", t );
					strcat( buf, string );
				}
				else
				{
					sprintf( string, "%lu ms", time );
					DivString( 3, 2, string );
					noDot = 0;
					strcat( buf, string );
				}
			}
			else
			{
				sprintf( string, "%lu us", time );
				strcat( buf, string );
			}
		}
	}
	else
	{
		if (!hours && !minutes)
		{
			if (!seconds) strcat( buf, "0" );
			strcat( buf, " s" );
		}
		return 0;
	}

	// remove trailing zeros
	len = strlen( buf );
	p = buf + len;

	// find the end of the number in string
	while(!isdigit( *p ) && p >= buf) p--;
	if (p < buf) return 0;							// no digit found

	if (noDot == 0)
	{
		// remove zero digits
		while (*p == '0' && p >= buf)
		{
			p1 = p;
			while (*p1 != '\0')
			{
				*p1 = *(p1 + 1);
				p1++;
			}
			p--;
		}

		// remove a dot, if last char
		if (*p == '.')
		{
			p1 = p;
			while (*p1 != '\0')
			{
				*p1 = *(p1 + 1);
				p1++;
			}
		}
	}
	else
	{
		p++;
		// remove a dot, if last char
		if (*p == '.')
		{
			p1 = p;
			while (*p1 != '\0')
			{
				*p1 = *(p1 + 1);
				p1++;
			}
		}
	}

	return 0;
}


//-----------------------------------------------------------------------------
// time in seconds

void MakeSecondString( char *buf, long time )
{
	long	hours, minutes;
	char	string[20];

	if (buf == NULL) return;
	*buf = '\0';

	hours = time / 3600UL;
	time -= hours * 3600UL;
	minutes = time / 60UL;
	time -= minutes * 60UL;

	if (hours)
	{
		sprintf( string, "%lu h ", hours );
		strcat( buf, string );
	}

	if (minutes)
	{
		sprintf( string, "%lu min ", minutes );
		strcat( buf, string );
	}
	else
	{
		if (hours)
		{
			strcat( buf, "0 min " );
		}
	}

	sprintf( string, "%lu sec", time );
	strcat( buf, string );
}


/**************************************************************
 *	Convert DOUBLE into scientific String, e.g. 0.01 -> "10 m"
 **************************************************************/

int Dbl2Str( char *str, const double dbl )
{
	int		result = 1;
	char	unit = '\0';
	int		sign = 0;
	double  d;

	if (str)
	{
		d = dbl;

		if (isinf(d))
		{
			sprintf( str, "inf " );
			result = 0;
			return result;
		}
		if (isnan(d))
		{
			sprintf( str, "NaN " );
			result = 0;
			return result;
		}

		if (d < 0.0)
		{
			d *= -1.0;
			sign = 1;
		}

		if (d < 1.0e-24)
		{
			unit = '\0';
			sign = 0;
		}
		else if (d < 1.0e-21)
		{
			d *= 1.0e24;
			unit = kYokto;
		}
		else if (d < 1.0e-18)
		{
			d *= 1.0e21;
			unit = kZepto;
		}
		else if (d < 1.0e-15)
		{
			d *= 1.0e18;
			unit = kAtto;
		}
		else if (d < 1.0e-12)
		{
			d *= 1.0e15;
			unit = kFemto;
		}
		else if (d < 1.0e-9)
		{
			d *= 1.0e12;
			unit = kPico;
		}
		else if (d < 1.0e-6)
		{
			d *= 1.0e9;
			unit = kNano;
		}
		else if (d < 1.0e-3)
		{
			d *= 1.0e6;
			unit = kMicro;
		}
		else if (d < 1.0)
		{
			d *= 1000.0;
			unit = kMilli;
		}
		else if (d < 1000.0)
		{
			unit = '\0';
		}
		else if (d < 1.0e6)
		{
			d /= 1000.0;
			unit = kKilo;
		}
		else if (d < 1.0e9)
//		else if (d < 999999999.999999)
		{
			d /= 1.0e6;
			unit = kMega;
		}
		else if (d < 1.0e12)
		{
			d /= 1.0e9;
			unit = kGiga;
		}
		else if (d < 1.0e15)
		{
			d /= 1.0e12;
			unit = kTera;
		}
		else if (d < 1.0e18)
		{
			d /= 1.0e15;
			unit = kPeta;
		}
		else if (d < 1.0e21)
		{
			d /= 1.0e18;
			unit = kExa;
		}
		else if (d < 1.0e24)
		{
			d /= 1.0e21;
			unit = kZetta;
		}
		else
		{
			d /= 1.0e24;
			unit = kYotta;
		}

		if (sign)
		{
			if (unit)
			{
				sprintf( str, "-%6.2f %c", d, unit );
			}
			else
			{
				sprintf( str, "-%6.2f ", d );
			}
			result = 0;
		}
		else
		{
			if (unit)
			{
				sprintf( str, "%6.2f %c", d, unit );
			}
			else
			{
				sprintf( str, "%6.2f ", d );
			}
			result = 0;
		}
		RemoveTrailingZeros( str );
	}

	return result;

} /* Dbl2Str */


/**************************************************************
 *	Convert scientific String into DOUBLE, e.g. "10m" -> 0.01
 **************************************************************/

int Str2Dbl( const char *str, double *dbl )
{
	if (str == NULL || dbl == NULL) return -1;
	if (*str == '\0') return 0;

	switch (str[0])
	{
		case kYokto:	*dbl /= 1.0e24;
						break;
		case kZepto:	*dbl /= 1.0e21;
						break;
		case kAtto:		*dbl /= 1.0e18;
						break;
		case kFemto:	*dbl /= 1.0e15;
						break;
		case kPico:		*dbl /= 1.0e12;
						break;
		case kNano:		*dbl /= 1.0e9;
						break;
		case kMicro:	*dbl /= 1.0e6;
						break;
		case kMilli:	*dbl /= 1000.0;
						break;
		case '%':
		case kCenti:	*dbl /= 100.0;
						break;
		case kDeci:		if (str[1] == 'a')
						{
							*dbl *= 10.0;
						}
						else
						{
							*dbl /= 10.0;
						}
						break;
		case kHekto:	*dbl *= 100.0;
						break;
		case kKilo:		*dbl *= 1000.0;
						break;
		case kMega:		*dbl *= 1.0e6;
						break;
		case kGiga:		*dbl *= 1.0e9;
						break;
		case kTera:		*dbl *= 1.0e12;
						break;
		case kPeta:		*dbl *= 1.0e15;
						break;
		case kExa:		*dbl *= 1.0e18;
						break;
		case kZetta:	*dbl *= 1.0e21;
						break;
		case kYotta:	*dbl *= 1.0e24;
						break;
	} /* switch */

	return 0;

} /* MyStr2Dbl */

//-----------------------------------------------------------------------------
int GetOptions( char *buf, char *list, char *argc, unsigned long *result )
{
	int				error = 1;
	unsigned long	shift, opt = 0;
	char			*ptr;
	char			num = 0;

	if (buf)
	{
		// skip leading spaces
		while (*buf == ' ' || *buf == '\t') buf++;

		// is there a hyphen
		if (*buf == '-')
		{
			if (isalpha( *(buf+1) ))
			{
				buf++;
				error = 0;
				while (isalpha( *buf ))
				{
					ptr = list;
					shift = 1;
					while (*ptr != '\0')
					{
						if (*buf == *ptr)
						{
							opt |= shift;
							num++;
							break;
						}
						ptr++;
						if (shift == 0x8000 && *ptr != '\0') break;
						shift <<= 1;
					}
					buf++;
				}
			}
		}
	}

	*argc = num;
	*result = opt;
	return error;
}


//-----------------------------------------------------------------------------
void SetPrompt( void )
{
	Prints( "\ndClamp -> " );
}

/*----------------------------------------------------------------------- *
	 Print the identifier
 * ----------------------------------------------------------------------- */
void PrintIdentifier( void )
{
//	unsigned long	result;
//	char			string[20];
	int				v1, v2, v3;

	v1 = VERSION / 100;
	v2 = (VERSION - v1 * 100) / 10;
	v3 = VERSION - v1 * 100 - v2 * 10;

//	GetBaudRate( SPI_AD_SPEED, pm_read_module_freq_hz( PM_PBA_SPI0 ), &result, string );
	Printf( " dynamic-clamp v%d.%d.%d (%s, %s) %d MHz\n",
				v1, v2, v3, __DATE__, __TIME__, pm_get_mclk_freq_hz() / 1000000 );
/*
	Printf( " dynamic-clamp v%d.%d.%d (%s, %s) CPU: %d MHz SPI: %s\n",
				v1, v2, v3, __DATE__, __TIME__, pm_get_mclk_freq_hz() / 1000000, string );
 	Printf( " PBA: %d\n", pm_read_module_freq_hz( PM_PBA_SPI0 ) );
 	Printf( " dynamic-clamp v0.1 (%s, %s) running at %d MHz\n",
				__DATE__, __TIME__, pm_get_mclk_freq_hz() / 1000000 );
	Printf( " SPI baudrate: %s\n", string );
*/
}


//-----------------------------------------------------------------------------
void PrintDivider( void )
{
	int		i;

	for (i = 0; i < 80; i++)
	{
		Prints( "-" );
	}
	Prints( "\n" );
}
// const char		*switches[] = 		{ "off", "on", NULL };

//-----------------------------------------------------------------------------
void Help( char *text )
{
	int		i, len;

	//               10        20        30        40        50        60        70        80        90
	//       123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890
	const char *helpTexts[] =
	{
			"-",
			"--system commands--",
			"help - this command",
			"version - prints the version of the program",
			"start {-rt} - starts the calculation -r run forever, -t trigger immedeately",
			"stop - stops the calculation",
			"gain {0.1}{1}{10}{100} - sets the output gain in pA/mV",
			"interval - sets the sampling interval",
			"get {-af} {aver.}{chan} - gets a value from ADC, -a average, -f run forever",
			"set {-dsb}[val] - sets DAC in Volt or -d DAC-Values -s 2. chan. -b both",
			"offset {chan}[val] - sets Offset in Volt",
			"clip {val1}{val2} - prints the times, where clipping occurred",
			"info - prints an info about the settings",
			"settings - prints all the settings",
			"ion {-h}{\"ion\"} - sets the ionchannel -h for help",
			"makelt - make lookup table",
			"date MMDDhhmm[[CC]YY][.ss] - sets the date and time",
			"open [datafile] - opens a datafile",
			"file [datafile] - same as open",
			"dump [address] {number} - prints the contents of the memory location",
			"firmware {-f} - loads the firmware to RAM -f flash after loading",
			"flash - writes the new firmware to the flash",
			"-",
			"--disk commands--",
			"mount [disk] - mount the drive (only possible is a:)",
			"ls - list directory",
			"cd [dirname] - change directory",
			"mkdir [dirname] - make a directory",
			"touch [filename] - make a file",
			"append [filename] - append text to a file until ^C",
			"cat [filename] - prints the content of a file",
			"disk - get number of drives",
			"df - get free space information",
			"fat - get FAT type for current drive",
//			"cp [filename] - copy filename to bookmark",
			"rm [filename] - erase file or EMPTY directory",
			"mv [src] [dst] move file or directory",
			"format [disk] - format the drive. For \"disk\", only \"a\" is used in the moment",
			"-",
			"--time commands--",
			"time {-a}[t1]{t2 ...} - sets the t0 array manually new or append (-a)",
			"time [-rp][first t][freq][num] - sets the t0 array -r regular or -p poisson",
			"maxtime {t} - sets the max time window for look up table",
			"-",
			"--alpha synapse commands--",
			"gaba_alpha {tpeak Gmax E} - sets the gaba alpha parameters",
			"tpeak {tpeak} - sets the t peak",
			"gmax {Gmax} - sets the G max",
			"erev {E} - sets the reversal potential",
			"-",
			"--2exp synapse commands--",
			"gaba_2exp {Gconst tauR tauD E} - sets the gaba 2exp parameters",
			"gconst {Gconst} - sets the G const",
			"taur {tauR} - sets the tau rising",
			"taud {tauD} - sets the tau decay",
			"erev {E} - sets the reversal potential",
			"-",
			"--dynamic synapse commands--",
			"dynamic {tpeak n pr sigma tauRec E gunit} - sets the dynamic alpha synapse",
			"tpeak {tpeak} - sets the t peak",
			"nsite {n} - sets the number of release sites",
			"prob {pr}- sets the mean release probability",
			"sigma {sigma} - sets the standard deviation of miniPSR",
			"taurec {tauRec} - set the recovery time constant",
			"tauf {tauF} - sets time constant for facilitation",
			"erev {E} - sets the reversal potential",
			"gunit {gunit} - sets the scale for g",
			"gdynamic - print out the Gmax values",
			"-",
			"--ampa synapse commands--",
			"ampa {gmax gsd tauR tauD E nsyn} - sets the ampa 2exp parameters",
			"gmax {gmax} - sets mean peak conductance",
			"gsd {gsd} - sets sd of peak conductance",
			"taur {tauR}- sets rising time constant",
			"taud {tauD} - sets decay time constant",
			"erev {E} - sets the reversal potential",
			"nsyn {nsyn} - sets the number of synapses",
			"-",
			"--testpulse commands--",
			"testpulse {G dt E t0} - sets the test pulse parameters",
			"gtp {G} - sets the conductance of the test pulse",
			"dttp {dt} - sets the width of the test pulse",
			"etp {E} - sets the voltage of the test pulse",
			"t0tp {t0} - sets the beginning of the test pulse",
			NULL
	};

	if (text[0] == '\0')
	{
		PrintIdentifier();
		for ( i = 0; helpTexts[i] != NULL; i++ )
		{
			if (helpTexts[i][0] == '-' && helpTexts[i][1] == '\0')
			{
				PrintDivider();
			}
			else
			{
				Prints( " ");
				Prints( helpTexts[i] );
				Prints( "\n");
			}
		}
	}
	else
	{
		len = strlen( text );
		for ( i = 0; helpTexts[i] != NULL; i++ )
		{
			if ((strncmp( helpTexts[i], text, len ) == 0))
			{
				Prints( " ");
				Prints( helpTexts[i] );
				Prints( "\n");
			}
		}
	}
}

/*
int GetHexFromChar( char *ch, int cnt )
{
	int		i, result = 0;

	for (i = 0; i < cnt; i++)
	{
		result <<= 4;
		if (ch[i] >= '0' && ch[i] <= '9')
		{
			result |= (ch[i] - '0');
		}
		else
		{
			ch[i] &= ~0x20;
			result |= (ch[i] - 'A' + 10);
		}
	}
	return result;
}
*/


int GetHexFromChar( char *ch, int cnt )
{
	int		i, result = 0;
	char	c;

	for (i = 0; i < cnt; i++)
	{
		c = ch[i];
		c -= '0';
		if (c >= 10)
		{
			c &= 0x1F;
			c -= 7;
		}
		result <<= 4;
		result |= c;
	}
	return result;
}


int CheckFile( void )
{
	volatile char	*ptr = (void *) BOARD_SDRAM_BASE;
	char			ch, type, bt[2];
	char			flag0 = 0, flag1 = 0, flag3 = 0, flag8 = 0;
	int				cnt, i, sum, val;

	// the first char must be a 'S'
//	if (*ptr != 'S') return NO_SREC;

	while (1)
	{
		cnt = 0;
		do
		{
			ch = *ptr++;
			if (cnt++ > 100) return NO_SREC;
		} while (ch != 'S');

		type = *ptr++;
		switch (type)
		{
			case '0':
				flag0 = 1;
				break;
			case '1':
				flag1 = 1;
				break;
			case '3':
				flag3 = 1;
				break;
			case '8':
				flag8 = 1;
				break;
		}
		if (type == '0') flag0 = 1;
		if (type == '1') flag1 = 1;

		bt[0] = *ptr++;
		bt[1] = *ptr++;
		cnt = GetHexFromChar( bt, 2 );

		sum = cnt--;
		for (i = 0; i < cnt; i++)
		{
			bt[0] = *ptr++;
			bt[1] = *ptr++;
			val = GetHexFromChar( bt, 2 );
			sum += val;
		}
		sum = ~sum;
		sum &= 0x000000FF;
		bt[0] = *ptr++;
		bt[1] = *ptr++;
		val = GetHexFromChar( bt, 2 );
		if (val != sum) return ERR_SREC;
		if (type >= '7') break;
	}
	if (flag0 == 0) return ERR_S0;
	if (flag1) return ERR_S1;
	if (flag3) return ERR_S3;
	if (flag8 == 0) return ERR_S8;
	return 0;
}


char GetNextChar( char type, int *cnt )
{
	volatile long	timeout = 10000UL;
	int				timeoutCnt = 2000;
	volatile int	wait;

	while ((GetBufferCnt()) == 0)
	{
		if (--timeout == 0)
		{
			if ((type >= '7' && *cnt <= 0) || --timeoutCnt == 0)
			{
//				Printf( "type = %c; cnt = %d; timeoutCnt = %d\n", type, *cnt, timeoutCnt );
				return 0;
			}
			timeout = 10000UL;
		}
		wait = 100;
		while (--wait);
	}
	(*cnt)--;
	return GetChar();
}


void LoadFile( void )
{
	volatile char	*ptr = (void *) BOARD_SDRAM_BASE;
	int				cnt = 0;
	int				bcnt = 0;
	char			ch, type = 0, bt[2];
	char			string[40];
	long			time;


	while ((GetBufferCnt()) == 0);			// wait for the first character

	time = gDate;
//	Prints( "load the file . . . " );

	while (1)
	{
		ch = GetNextChar( type, &bcnt );
		*ptr++ = ch;

		if (ch == 0) break;

		if (ch == 'S')
		{
//			if ((cnt & 3) == 3) Prints( "." );
			Prints( "." );
			if (++cnt >= 80)
			{
				Prints( "\n" );
				cnt = 0;
			}

			ch = GetNextChar( type, &bcnt );
			*ptr++ = ch;
			bt[0] = GetNextChar( type, &bcnt );
			*ptr++ = bt[0];
			bt[1] = GetNextChar( type, &bcnt );
			*ptr++ = bt[1];
			bcnt = GetHexFromChar( bt, 2 );
			bcnt <<= 1;
			type = ch;

		}
	}
//	Prints( " ready\n" );
//	Printf( "type = %c; bcnt = %d\n", type, bcnt );
	time = gDate - time;
	MakeSecondString( string, time );
	Printf( "\nloading took: %s\n", string );

}


/*
int LoadFile( void )
{
	volatile char			*ptr = (void *) BOARD_SDRAM_BASE;
	volatile unsigned long	timeout;
	int						error = 0, cnt = 0, sFlag = 0, sum = 0, scnt = 0, wait = 0;
	char					ch, type = 0;


	while ((GetBufferCnt()) == 0);			// wait for the first character

	while (1)
	{
		timeout = 20000000UL;
		while ((GetBufferCnt()) == 0)
		{
			if (--timeout == 0)
			{
				error = ERR_TIMEOUT;
				break;
			}
			wait++;
		}
		if (scnt < 100)
		{
			sum += wait;
			wait = 0;
			scnt++;
		}
		else if (scnt == 100)
		{
			sum /= scnt;
			scnt++;
		}

		if (error) break;
		ch = GetChar();
		*ptr++ = ch;
//		ptr++;
		if (sFlag)
		{
			sFlag = 0;
			type = ch;
		}

		if (ch == 'S')
		{
			sFlag = 1;
			Prints( "." );
			if (cnt++ > 80)
			{
				Prints( "\n" );
				cnt = 0;
			}
		}
	}
	Prints( "\n" );
//	Printf( "RAM end address: 0x%08X\n", ptr );
	Printf( "mean wait count = %d\n", sum );

	return error;
}
*/



